﻿using System;
using System.Collections.Generic;

using System.Text;
using Aquiles.Core.Model;
using Apache.Cassandra;
using Aquiles.Core.Exceptions;
using System.Globalization;
using Aquiles.Cassandra07.Converter;
using Aquiles.Cassandra07.Model;

namespace Aquiles.Cassandra07.Command
{
    /// <summary>
    /// Command to batch actions over a Keyspace of a Cluster.
    /// Posible actions are:
    ///     - insert
    ///     - delete
    /// </summary>
    public class BatchMutateCommand : AbstractKeyspaceDependantCommand, IAquilesCommand
    {
        /// <summary>
        /// get or set Mutation actions to be applied
        /// </summary>
        public Dictionary<byte[], Dictionary<string, List<IAquilesMutation>>> Mutations
        {
            get;
            set;
        }

        #region IAquilesCommand Members
        /// <summary>
        /// Executes a "batch_mutate" over the connection. No return values
        /// </summary>
        /// <param name="cassandraClient">opened Thrift client</param>
        public void Execute(Cassandra.Client cassandraClient)
        {
            Dictionary<byte[], Dictionary<string, List<Mutation>>> mutation_map = this.translate(this.Mutations);
            cassandraClient.batch_mutate(mutation_map, this.GetCassandraConsistencyLevel());
        }

        /// <summary>
        /// Validate the input parameters. 
        /// Throws <see cref="Aquiles.Core.Exceptions.AquilesException"/>  in case there is some malformed or missing input parameters
        /// </summary>
        public void ValidateInput()
        {
            if (this.Mutations != null)
            {
                byte[] key;
                string columnFamily;
                List<IAquilesMutation> mutations;
                Dictionary<string, List<IAquilesMutation>> mutationsOverAKey;
                Dictionary<string, List<IAquilesMutation>>.Enumerator mutationsOverAKeyEnumerator;
                Dictionary<byte[], Dictionary<string, List<IAquilesMutation>>>.Enumerator keysEnumerator = this.Mutations.GetEnumerator();
                while (keysEnumerator.MoveNext())
                {
                    key = keysEnumerator.Current.Key;
                    if (key != null && key.Length != 0)
                    {
                        mutationsOverAKey = keysEnumerator.Current.Value;
                        mutationsOverAKeyEnumerator = mutationsOverAKey.GetEnumerator();
                        while (mutationsOverAKeyEnumerator.MoveNext())
                        {
                            columnFamily = mutationsOverAKeyEnumerator.Current.Key;
                            if (!String.IsNullOrEmpty(columnFamily))
                            {
                                mutations = mutationsOverAKeyEnumerator.Current.Value;
                                if ((mutations == null) || (mutations != null && mutations.Count == 0))
                                {
                                    throw new AquilesException(String.Format(CultureInfo.CurrentCulture, "No mutations found for ColumnFamily '{0}' over Key '{1}'.", columnFamily, key));
                                }
                                else
                                {
                                    foreach (IAquilesMutation mutation in mutations)
                                    {
                                        mutation.Validate();
                                    }
                                }
                            }
                            else
                            {
                                throw new AquilesException("ColumnFamily cannot be null or empty.");
                            }
                        }
                    }
                    else
                    {
                        throw new AquilesException("Key cannot be null or empty.");
                    }
                }
            }
            else
            {
                throw new AquilesException("No mutations found");
            }
        }

        #endregion

        private Dictionary<byte[], Dictionary<string, List<Mutation>>> translate(Dictionary<byte[], Dictionary<string, List<IAquilesMutation>>> mutations)
        {
            Dictionary<string, List<IAquilesMutation>>.Enumerator mutationOverKeyEnumerator;
            Dictionary<byte[], Dictionary<string, List<Mutation>>> cassandraKeyColumnFamilyMutationMap = new Dictionary<byte[], Dictionary<string, List<Mutation>>>();
            Dictionary<string, List<Mutation>> cassandraColumnFamilyMutationMap = null;
            List<Mutation> mutationList = null;
            Dictionary<byte[], Dictionary<string, List<IAquilesMutation>>>.Enumerator mutationEnumerator = mutations.GetEnumerator();
            while (mutationEnumerator.MoveNext())
            {
                cassandraColumnFamilyMutationMap = new Dictionary<string, List<Mutation>>();
                mutationOverKeyEnumerator = mutationEnumerator.Current.Value.GetEnumerator();
                while (mutationOverKeyEnumerator.MoveNext())
                {
                    mutationList = new List<Mutation>(mutationOverKeyEnumerator.Current.Value.Count);
                    foreach (IAquilesMutation aquilesMutation in mutationOverKeyEnumerator.Current.Value)
                    {
                        mutationList.Add(ModelConverterHelper.Convert<IAquilesMutation,Mutation>(aquilesMutation));
                    }
                    cassandraColumnFamilyMutationMap.Add(mutationOverKeyEnumerator.Current.Key, mutationList);
                }
                cassandraKeyColumnFamilyMutationMap.Add(mutationEnumerator.Current.Key, cassandraColumnFamilyMutationMap);
            }

            return cassandraKeyColumnFamilyMutationMap;
        }
    }
}