﻿using System;
using System.Collections.Generic;

using System.Text;
using Apache.Cassandra;
using Aquiles.Core.Exceptions;

namespace Aquiles.Cassandra07.Model
{
    /// <summary>
    /// Aquiles structure to contains Cassandra Key Range structure (only support for a key range get)
    /// <remarks>
    /// StartKey will be included in the response
    /// The semantics of start keys and tokens are slightly different. Keys are start-inclusive; tokens are start-exclusive. Token ranges may also wrap -- that is, the end token may be less than the start one. Thus, a range from keyX to keyX is a one-element range, but a range from tokenY to tokenY is the full ring. 
    /// </remarks>
    /// </summary>
    public class AquilesKeyRange : IAquilesObject
    {
        /// <summary>
        /// get or set the Start Key (included in the command response)
        /// </summary>
        public byte[] StartKey
        {
            get;
            set;
        }

        /// <summary>
        /// get or set the Endkey
        /// </summary>
        public byte[] EndKey
        {
            get;
            set;
        }

        /// <summary>
        /// get or set the Start Token (excluded in the command response)
        /// </summary>
        public string StartToken
        {
            get;
            set;
        }

        /// <summary>
        /// get or set the End Token
        /// </summary>
        public string EndToken
        {
            get;
            set;
        }

        #region IAquilesObject<KeyRange> Members

        /// <summary>
        /// get or set how many keys to permit in the KeyRange
        /// </summary>
        public int Count
        {
            get;
            set;
        }
        
        /// <summary>
        /// Validate the object data to assure consistency when used as input parameter when used in an insert Operation
        /// <remarks>Throw <see cref="Aquiles.Core.Exceptions.AquilesException"/> in case there is something wrong</remarks>
        /// </summary>
        public void ValidateForInsertOperation()
        {
            this.Validate();
        }

        /// <summary>
        /// Validate the object data to assure consistency when used as input parameter when used in an deletation Operation
        /// <remarks>Throw <see cref="Aquiles.Core.Exceptions.AquilesException"/> in case there is something wrong</remarks>
        /// </summary>
        public void ValidateForDeletationOperation()
        {
            this.Validate();
        }

        /// <summary>
        /// Validate the object data to assure consistency when used as input parameter when used in an set / update Operation
        /// <remarks>Throw <see cref="Aquiles.Core.Exceptions.AquilesException"/> in case there is something wrong</remarks>
        /// </summary>
        public void ValidateForSetOperation()
        {
            this.Validate();
        }

        /// <summary>
        /// Validate the object data to assure consistency when used as input parameter when used in an set / update Operation
        /// <remarks>Throw <see cref="Aquiles.Core.Exceptions.AquilesException"/> in case there is something wrong</remarks>
        /// </summary>
        public void ValidateForQueryOperation()
        {
            this.Validate();
        }

        private void Validate() 
        {
            this.ValidateCountGreaterThanZero();
            if (this.StartKey != null || this.EndKey != null)
            {
                this.ValidateNullOrEmptyStartKey();
                this.ValidateNullOrEmptyEndKey();
            }
            else if (!String.IsNullOrEmpty(this.StartToken) || !String.IsNullOrEmpty(this.EndToken))
            {
                this.ValidateNullOrEmptyStartToken();
                this.ValidateNullOrEmptyEndToken();
            }
            else
            {
                throw new AquilesException("Neither keys not tokens specified.");
            }
        }

        private void ValidateNullOrEmptyEndToken()
        {
            if (String.IsNullOrEmpty(this.EndToken))
            {
                throw new AquilesException("End Token must be not null or empty.");
            }
        }

        private void ValidateNullOrEmptyStartToken()
        {
            if (String.IsNullOrEmpty(this.StartToken))
            {
                throw new AquilesException("Start Token must be not null or empty.");
            }
        }

        private void ValidateNullOrEmptyEndKey()
        {
            if (this.EndKey == null)
            {
                throw new AquilesException("EndKey must not be null.");
            }
        }

        private void ValidateNullOrEmptyStartKey()
        {
            if (this.StartKey == null)
            {
                throw new AquilesException("StartKey must not be null.");
            }
        }

        private void ValidateCountGreaterThanZero()
        {
            if (this.Count <= 0)
            {
                throw new AquilesException("Quantity must be greater than 0.");
            }
        }

        #endregion
    }
}
